from abc import ABC, abstractmethod


class Version(ABC):
    def __init__(self, raw: str):
        self.raw = raw

    def __repr__(self):
        return self.raw

    @abstractmethod
    def is_pre_release(self) -> bool:
        raise NotImplementedError

    @abstractmethod
    def __eq__(self, other: object) -> bool:
        raise NotImplementedError

    @abstractmethod
    def __lt__(self, other: object) -> bool:
        raise NotImplementedError



from kataloger.exceptions.kataloger_exception import KatalogerException


class KatalogerConfigurationException(KatalogerException):
    def __init__(self, message: str):
        super().__init__(message)



from abc import ABC, abstractmethod
from typing import List, Optional, Tuple

from kataloger.data.artifact.artifact import Artifact
from kataloger.data.artifact_update import ArtifactUpdate
from kataloger.data.metadata_repository_info import MetadataRepositoryInfo
from kataloger.update_resolver.base.update_resolution import UpdateResolution


class UpdateResolver(ABC):

    @abstractmethod
    def resolve(
        self,
        artifact: Artifact,
        repositories_metadata: List[MetadataRepositoryInfo],
    ) -> Tuple[UpdateResolution, Optional[ArtifactUpdate]]:
        raise NotImplementedError



from kataloger.data.artifact.artifact import Artifact


class Plugin(Artifact):
    def __init__(self, name: str, coordinates: str, version: str):
        super().__init__(name, coordinates, version)

    def to_path(self) -> str:
        return f"{self.coordinates.replace('.', '/')}/{self.coordinates}.gradle.plugin"



from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import Optional


@dataclass(frozen=True)
class Artifact(ABC):
    name: Optional[str]
    coordinates: str
    version: str

    def __repr__(self):
        return self.name

    @abstractmethod
    def to_path(self) -> str:
        pass



from dataclasses import dataclass
from pathlib import Path
from typing import Optional

from kataloger.helpers.backport_helpers import remove_suffix


@dataclass(frozen=True)
class Catalog:
    name: Optional[str]
    path: Path

    @staticmethod
    def from_path(path: Path) -> "Catalog":
        return Catalog(
            name=remove_suffix(path.name, ".versions.toml"),
            path=path,
        )



from dataclasses import dataclass
from typing import List

from kataloger.data.catalog import Catalog
from kataloger.data.repository import Repository


@dataclass(frozen=True)
class KatalogerConfiguration:
    catalogs: List[Catalog]
    library_repositories: List[Repository]
    plugin_repositories: List[Repository]
    verbose: bool
    suggest_unstable_updates: bool
    fail_on_updates: bool



from kataloger.update_resolver.universal.universal_version import UniversalVersion
from kataloger.update_resolver.universal.version_factory import VersionFactory


class UniversalVersionFactory(VersionFactory[UniversalVersion]):

    def create(self, version: str) -> UniversalVersion:
        return UniversalVersion(version)

    def can_create(self, version: str) -> bool:
        return UniversalVersion.can_handle(version)



from dataclasses import dataclass

from kataloger.data.artifact_metadata import ArtifactMetadata
from kataloger.data.repository import Repository


@dataclass(frozen=True)
class MetadataRepositoryInfo:
    repository: Repository
    metadata: ArtifactMetadata



import asyncio
from collections import defaultdict
from typing import Dict, List, Optional

from aiohttp import BasicAuth, ClientSession

from kataloger.data.artifact.artifact import Artifact
from kataloger.data.metadata_repository_info import MetadataRepositoryInfo
from kataloger.data.repository import Repository
from kataloger.helpers.log_helpers import log_warning
from kataloger.helpers.xml_parse_helpers import try_parse_maven_group_metadata


async def get_all_artifact_metadata(
    artifacts: List[Artifact],
    repositories: List[Repository],
    verbose: bool,
) -> Dict[Artifact, List[MetadataRepositoryInfo]]:
    if not artifacts:
        return {}

    search_results: Dict[Artifact, List[MetadataRepositoryInfo]] = defaultdict(list)
    for repository in repositories:
        result = await get_all_artifact_metadata_in_repository(repository, artifacts, verbose)
        for artifact, metadata in result.items():
            search_results[artifact].append(metadata)
    return search_results


async def get_all_artifact_metadata_in_repository(
    repository: Repository,
    artifacts: List[Artifact],
    verbose: bool,
) -> Dict[Artifact, MetadataRepositoryInfo]:
    if repository.requires_authorization():
        auth = BasicAuth(login=repository.user, password=repository.password)
    else:
        auth = None

    async with ClientSession(auth=auth) as session:
        requests = []
        for artifact in artifacts:
            request = get_artifact_metadata(session, repository, artifact, verbose)
            requests.append(request)
        results = await asyncio.gather(*requests)

    return {artifact: metadata for artifact, metadata in zip(artifacts, results) if metadata}


async def get_artifact_metadata(
    session: ClientSession,
    repository: Repository,
    artifact: Artifact,
    verbose: bool,
) -> Optional[MetadataRepositoryInfo]:
    metadata_url = repository.address / artifact.to_path() / "maven-metadata.xml"
    async with session.get(metadata_url) as response:
        if response.status != 200:
            return None

        metadata = try_parse_maven_group_metadata(await response.text())
        if not metadata and verbose:
            log_warning(f"Can't parse metadata for {artifact.name} in {repository.name}.")
        return MetadataRepositoryInfo(repository, metadata)



from kataloger.data.artifact.artifact import Artifact


class Library(Artifact):
    def __init__(self, name: str, coordinates: str, version: str):
        super().__init__(name, coordinates, version)

    def to_path(self) -> str:
        return self.coordinates.replace(".", "/").replace(":", "/")



import sys
from pathlib import Path
from typing import List, Optional, Tuple, TypeVar

from kataloger.cli.argument_parser import parse_arguments
from kataloger.data.catalog import Catalog
from kataloger.data.configuration_data import ConfigurationData
from kataloger.data.kataloger_arguments import KatalogerArguments
from kataloger.data.kataloger_configuration import KatalogerConfiguration
from kataloger.data.repository import Repository
from kataloger.exceptions.kataloger_configuration_exception import KatalogerConfigurationException
from kataloger.helpers.backport_helpers import get_package_file
from kataloger.helpers.path_helpers import file_exists
from kataloger.helpers.toml_parse_helpers import load_configuration

T = TypeVar("T")


def merge(
    first: Optional[T],
    second: Optional[T],
    default: T,
) -> T:
    if first:
        return first
    if second:
        return second

    return default


def get_configuration() -> KatalogerConfiguration:
    arguments: KatalogerArguments = parse_arguments(*sys.argv[1:])
    args_cd: ConfigurationData = arguments.configuration_data
    conf_cd: ConfigurationData = load_configuration_data(arguments.configuration_path)

    catalogs: List[Catalog] = get_catalogs(args_cd.catalogs, conf_cd.catalogs)
    library_repositories: List[Repository]
    plugin_repositories: List[Repository]
    library_repositories, plugin_repositories = get_repositories(
        arg_library_repositories=args_cd.library_repositories,
        arg_plugin_repositories=args_cd.plugin_repositories,
        conf_library_repositories=conf_cd.library_repositories,
        conf_plugin_repositories=conf_cd.plugin_repositories,
    )

    return KatalogerConfiguration(
        catalogs=catalogs,
        library_repositories=library_repositories,
        plugin_repositories=plugin_repositories,
        verbose=merge(args_cd.verbose, conf_cd.verbose, default=False),
        suggest_unstable_updates=merge(
            args_cd.suggest_unstable_updates,
            conf_cd.suggest_unstable_updates,
            default=False,
        ),
        fail_on_updates=merge(args_cd.fail_on_updates, conf_cd.fail_on_updates, default=False),
    )


def get_catalogs(arg_catalogs: Optional[List[Catalog]], conf_catalogs: Optional[List[Catalog]]) -> List[Catalog]:
    if arg_catalogs:
        return arg_catalogs

    if conf_catalogs:
        return conf_catalogs

    # If catalogs not provided via command line arguments or specified in configuration trying to find them in cwd.
    cwd_catalogs: Optional[List[Catalog]] = find_cwd_catalogs()
    if cwd_catalogs:
        return cwd_catalogs

    message = ("Gradle version catalog not found in current directory. "
               "Please specify path to catalog via parameter, in configuration "
               "file or run tool from directory with catalog (*.versions.toml) file.")
    raise KatalogerConfigurationException(message)


def get_repositories(
    arg_library_repositories: Optional[List[Repository]],
    arg_plugin_repositories: Optional[List[Repository]],
    conf_library_repositories: Optional[List[Repository]],
    conf_plugin_repositories: Optional[List[Repository]],
) -> Tuple[List[Repository], List[Repository]]:
    library_repositories: List[Repository]
    plugin_repositories: List[Repository]
    if arg_library_repositories or arg_plugin_repositories:
        library_repositories = arg_library_repositories if arg_library_repositories is not None else []
        plugin_repositories = arg_plugin_repositories if arg_plugin_repositories is not None else []
        return library_repositories, plugin_repositories

    if conf_library_repositories or conf_plugin_repositories:
        library_repositories = conf_library_repositories if conf_library_repositories is not None else []
        plugin_repositories = conf_plugin_repositories if conf_plugin_repositories is not None else []
        return library_repositories, plugin_repositories

    message = ("No repositories provided! You can specify repositories to "
               "search artifact updates through configuration file.")
    raise KatalogerConfigurationException(message)


def find_cwd_catalogs() -> Optional[List[Catalog]]:
    catalog_files = Path.cwd().glob("*.versions.toml")
    catalog_paths = filter(file_exists, catalog_files)
    if not catalog_paths:
        return None

    return [Catalog.from_path(path) for path in catalog_paths]


def load_configuration_data(configuration_path: Optional[Path]) -> ConfigurationData:
    if not configuration_path:
        configuration_candidate = Path.cwd() / "default.configuration.toml"
        if file_exists(configuration_candidate):
            configuration_path = configuration_candidate
        else:
            configuration_path = get_package_file("default.configuration.toml")

    return load_configuration(configuration_path)



from typing import List, Optional
from unittest.mock import AsyncMock, Mock, call, patch

import pytest

from entity_factory import EntityFactory
from kataloger.catalog_updater import CatalogUpdater
from kataloger.data.artifact.library import Library
from kataloger.data.artifact.plugin import Plugin
from kataloger.data.artifact_update import ArtifactUpdate
from kataloger.data.repository import Repository
from kataloger.exceptions.kataloger_configuration_exception import KatalogerConfigurationException
from kataloger.update_resolver.base.update_resolution import UpdateResolution
from kataloger.update_resolver.base.update_resolver import UpdateResolver


class TestCatalogUpdater:

    def test_should_raise_configuration_exception_when_no_library_and_plugin_repositories_provided(self):
        with pytest.raises(KatalogerConfigurationException, match="No repositories provided!"):
            self._create_catalog_updater(
                library_repositories=[],
                plugin_repositories=[],
                update_resolvers=[Mock()],
            )

    def test_should_not_raise_configuration_exception_when_no_library_repositories_provided(self):
        repository: Repository = EntityFactory.create_repository()
        self._create_catalog_updater(
            library_repositories=[],
            plugin_repositories=[repository],
            update_resolvers=[Mock()],
        )

    def test_should_not_raise_configuration_exception_when_no_plugin_repositories_provided(self):
        repository: Repository = EntityFactory.create_repository()
        self._create_catalog_updater(
            library_repositories=[repository],
            plugin_repositories=[],
            update_resolvers=[Mock()],
        )

    def test_should_raise_configuration_exception_when_no_update_resolvers_provided(self):
        repository: Repository = EntityFactory.create_repository()
        with pytest.raises(KatalogerConfigurationException, match="No update resolvers provided!"):
            self._create_catalog_updater(
                library_repositories=[repository],
                plugin_repositories=[repository],
                update_resolvers=[],
            )

    def test_try_find_update_should_return_none_when_resolver_update_resolution_is_no_updates(self):
        library: Library = EntityFactory.create_library()
        repository: Repository = EntityFactory.create_repository()
        resolver_mock: Mock = self._create_resolver_mock(resolution=UpdateResolution.NO_UPDATES)

        catalog_updater: CatalogUpdater = self._create_catalog_updater(
            library_repositories=[repository],
            update_resolvers=[resolver_mock],
        )
        actual_update: ArtifactUpdate = catalog_updater.try_find_update(artifact=library, repositories_metadata=[])

        assert actual_update is None
        resolver_mock.resolve.assert_called_once()

    def test_try_find_update_should_return_update_when_resolver_update_resolution_is_update_found(self):
        library: Library = EntityFactory.create_library()
        repository: Repository = EntityFactory.create_repository()
        expected_update: ArtifactUpdate = EntityFactory.create_artifact_update(
            name=library.name,
            update_repository_name=repository.name,
        )
        resolver_mock: Mock = self._create_resolver_mock(UpdateResolution.UPDATE_FOUND, expected_update)

        catalog_updater: CatalogUpdater = self._create_catalog_updater(
            library_repositories=[repository],
            update_resolvers=[resolver_mock],
        )
        actual_update: ArtifactUpdate = catalog_updater.try_find_update(artifact=library, repositories_metadata=[])

        assert actual_update == expected_update
        resolver_mock.resolve.assert_called_once()

    def test_try_find_update_should_return_update_when_first_resolver_cant_resolve_update_but_second_does(self):
        library: Library = EntityFactory.create_library()
        repository: Repository = EntityFactory.create_repository()
        metadata_mock: List[Mock] = [Mock()]
        expected_update: ArtifactUpdate = EntityFactory.create_artifact_update(
            name=library.name,
            update_repository_name=repository.name,
        )
        first_resolver: Mock = self._create_resolver_mock(UpdateResolution.CANT_RESOLVE)
        second_resolver: Mock = self._create_resolver_mock(UpdateResolution.UPDATE_FOUND, expected_update)

        catalog_updater: CatalogUpdater = self._create_catalog_updater(
            library_repositories=[repository],
            update_resolvers=[first_resolver, second_resolver],
        )
        actual_update: ArtifactUpdate = catalog_updater.try_find_update(
            artifact=library,
            repositories_metadata=metadata_mock,
        )

        assert actual_update == expected_update
        first_resolver.resolve.assert_called_once_with(library, metadata_mock)
        second_resolver.resolve.assert_called_once_with(library, metadata_mock)

    @pytest.mark.asyncio()
    async def test_get_library_updates_should_return_no_updates_when_there_is_no_library_repositories(self):
        library: Library = EntityFactory.create_library()
        repository: Repository = EntityFactory.create_repository()
        resolver_mock: Mock = self._create_resolver_mock(UpdateResolution.NO_UPDATES)
        catalog_updater: CatalogUpdater = self._create_catalog_updater(
            plugin_repositories=[repository],
            update_resolvers=[resolver_mock],
        )
        actual_updates: List[ArtifactUpdate] = await catalog_updater.get_library_updates(libraries=[library])

        assert actual_updates == []
        resolver_mock.resolve.assert_not_called()

    @pytest.mark.asyncio()
    async def test_get_library_updates_should_get_metadata_for_libraries_and_return_not_none_updates(self):
        library: Library = EntityFactory.create_library()
        repository: Repository = EntityFactory.create_repository()
        expected_update: ArtifactUpdate = EntityFactory.create_artifact_update(
            name=library.name,
            update_repository_name=repository.name,
        )
        libraries = [library, library]
        resolver_mock: Mock = Mock()
        resolver_mock.resolve.side_effect = [
            (UpdateResolution.NO_UPDATES, None),
            (UpdateResolution.UPDATE_FOUND, expected_update),
        ]
        catalog_updater: CatalogUpdater = self._create_catalog_updater(
            library_repositories=[repository],
            update_resolvers=[resolver_mock],
        )
        repositories_metadata = {Mock(): Mock(), Mock(): Mock()}
        with patch(
            target="kataloger.catalog_updater.get_all_artifact_metadata",
            new=AsyncMock(return_value=repositories_metadata),
        ) as load_metadata_mock:
            actual_updates: List[ArtifactUpdate] = await catalog_updater.get_library_updates(libraries)

        assert actual_updates == [expected_update]
        assert resolver_mock.resolve.call_count == 2
        load_metadata_mock.assert_called_once_with(
            artifacts=libraries,
            repositories=[repository],
            verbose=False,
        )

    @pytest.mark.asyncio()
    async def test_get_plugin_updates_should_return_no_updates_when_there_is_no_plugin_repositories(self):
        plugin: Plugin = EntityFactory.create_plugin()
        repository: Repository = EntityFactory.create_repository()
        resolver_mock: Mock = self._create_resolver_mock(UpdateResolution.NO_UPDATES)
        catalog_updater: CatalogUpdater = self._create_catalog_updater(
            library_repositories=[repository],
            update_resolvers=[resolver_mock],
        )
        actual_updates: List[ArtifactUpdate] = await catalog_updater.get_plugin_updates(plugins=[plugin])

        assert actual_updates == []
        resolver_mock.resolve.assert_not_called()

    @pytest.mark.asyncio()
    async def test_get_plugin_updates_should_get_metadata_for_plugins_and_return_not_none_updates(self):
        plugin: Plugin = EntityFactory.create_plugin()
        repository: Repository = EntityFactory.create_repository()
        expected_update: ArtifactUpdate = EntityFactory.create_artifact_update(
            name=plugin.name,
            update_repository_name=repository.name,
        )
        plugins = [plugin, plugin]
        resolver_mock: Mock = Mock()
        resolver_mock.resolve.side_effect = [
            (UpdateResolution.NO_UPDATES, None),
            (UpdateResolution.UPDATE_FOUND, expected_update),
        ]
        catalog_updater: CatalogUpdater = self._create_catalog_updater(
            plugin_repositories=[repository],
            update_resolvers=[resolver_mock],
        )
        repositories_metadata = {Mock(): Mock(), Mock(): Mock()}
        with patch(
            target="kataloger.catalog_updater.get_all_artifact_metadata",
            new=AsyncMock(return_value=repositories_metadata),
        ) as load_metadata_mock:
            actual_updates: List[ArtifactUpdate] = await catalog_updater.get_plugin_updates(plugins)

        assert actual_updates == [expected_update]
        assert resolver_mock.resolve.call_count == 2
        load_metadata_mock.assert_called_once_with(
            artifacts=plugins,
            repositories=[repository],
            verbose=False,
        )

    @pytest.mark.asyncio()
    async def test_get_updates_should_return_updates_for_libraries_and_plugins(self):
        library: Library = EntityFactory.create_library()
        plugin: Plugin = EntityFactory.create_plugin()
        library_repository: Repository = EntityFactory.create_repository(name="library_repository")
        plugin_repository: Repository = EntityFactory.create_repository(name="plugin_repository")
        library_update: ArtifactUpdate = EntityFactory.create_artifact_update(
            name=library.name,
            update_repository_name=library_repository.name,
        )
        plugin_update: ArtifactUpdate = EntityFactory.create_artifact_update(
            name=plugin.name,
            update_repository_name=plugin_repository.name,
        )
        resolver_mock: Mock = Mock()
        resolver_mock.resolve.side_effect = [
            (UpdateResolution.UPDATE_FOUND, library_update),
            (UpdateResolution.UPDATE_FOUND, plugin_update),
        ]
        catalog_updater: CatalogUpdater = self._create_catalog_updater(
            library_repositories=[library_repository],
            plugin_repositories=[plugin_repository],
            update_resolvers=[resolver_mock],
        )

        with patch(
            target="kataloger.catalog_updater.get_all_artifact_metadata",
            new=AsyncMock(return_value={Mock(): Mock()}),
        ) as load_metadata_mock:
            library_updates, plugin_updates = await catalog_updater.get_updates(libraries=[library], plugins=[plugin])

        assert library_updates == [library_update]
        assert plugin_updates == [plugin_update]
        assert resolver_mock.resolve.call_count == 2
        expected_load_metadata_calls = [
            call(
                artifacts=[library],
                repositories=[library_repository],
                verbose=False,
            ),
            call(
                artifacts=[plugin],
                repositories=[plugin_repository],
                verbose=False,
            ),
        ]
        assert load_metadata_mock.call_args_list == expected_load_metadata_calls

    @pytest.mark.asyncio()
    async def test_get_artifact_updates_should_return_artifact_updates_from_correct_repositories(self):
        library: Library = EntityFactory.create_library()
        plugin: Plugin = EntityFactory.create_plugin()
        library_repository: Repository = EntityFactory.create_repository(name="library_repository")
        plugin_repository: Repository = EntityFactory.create_repository(name="plugin_repository")
        library_update: ArtifactUpdate = EntityFactory.create_artifact_update(
            name=library.name,
            update_repository_name=library_repository.name,
        )
        plugin_update: ArtifactUpdate = EntityFactory.create_artifact_update(
            name=plugin.name,
            update_repository_name=plugin_repository.name,
        )
        resolver_mock: Mock = Mock()
        resolver_mock.resolve.side_effect = [
            (UpdateResolution.UPDATE_FOUND, library_update),
            (UpdateResolution.UPDATE_FOUND, plugin_update),
        ]
        catalog_updater: CatalogUpdater = self._create_catalog_updater(
            library_repositories=[library_repository],
            plugin_repositories=[plugin_repository],
            update_resolvers=[resolver_mock],
        )

        with patch(
            target="kataloger.catalog_updater.get_all_artifact_metadata",
            new=AsyncMock(return_value={Mock(): Mock()}),
        ) as load_metadata_mock:
            artifact_updates: List[ArtifactUpdate] = await catalog_updater.get_artifact_updates(
                artifacts=[library, plugin],
            )

        assert artifact_updates == [library_update, plugin_update]
        assert resolver_mock.resolve.call_count == 2
        expected_load_metadata_calls = [
            call(
                artifacts=[library],
                repositories=[library_repository],
                verbose=False,
            ),
            call(
                artifacts=[plugin],
                repositories=[plugin_repository],
                verbose=False,
            ),
        ]
        assert load_metadata_mock.call_args_list == expected_load_metadata_calls

    @pytest.mark.asyncio()
    async def test_should_return_empty_list_when_there_are_no_libraries_and_plugins_in_loaded_catalog(self):
        expected_updates: List[ArtifactUpdate] = []
        repository: Repository = EntityFactory.create_repository()
        catalog_updater: CatalogUpdater = self._create_catalog_updater(
            library_repositories=[repository],
            plugin_repositories=[repository],
            update_resolvers=[Mock()],
        )
        with patch("kataloger.catalog_updater.load_catalog", Mock(return_value=([], []))):
            actual_updates: List[ArtifactUpdate] = await catalog_updater.get_catalog_updates(catalog_path=Mock())

        assert actual_updates == expected_updates

    @pytest.mark.asyncio()
    async def test_should_return_artifact_updates_when_there_are_libraries_and_plugins_in_loaded_catalog(self):
        library: Library = EntityFactory.create_library()
        plugin: Plugin = EntityFactory.create_plugin()
        library_repository: Repository = EntityFactory.create_repository(name="library_repository")
        plugin_repository: Repository = EntityFactory.create_repository(name="plugin_repository")
        library_update: ArtifactUpdate = EntityFactory.create_artifact_update(
            name=library.name,
            update_repository_name=library_repository.name,
        )
        plugin_update: ArtifactUpdate = EntityFactory.create_artifact_update(
            name=plugin.name,
            update_repository_name=plugin_repository.name,
        )
        resolver_mock: Mock = Mock()
        resolver_mock.resolve.side_effect = [
            (UpdateResolution.UPDATE_FOUND, library_update),
            (UpdateResolution.UPDATE_FOUND, plugin_update),
        ]
        catalog_updater: CatalogUpdater = self._create_catalog_updater(
            library_repositories=[library_repository],
            plugin_repositories=[plugin_repository],
            update_resolvers=[resolver_mock],
        )

        load_metadata_mock = AsyncMock(return_value={Mock(): Mock()})
        with patch(target="kataloger.catalog_updater.load_catalog", new=Mock(return_value=([library], [plugin]))), \
             patch(target="kataloger.catalog_updater.get_all_artifact_metadata", new=load_metadata_mock):
            artifact_updates: List[ArtifactUpdate] = await catalog_updater.get_catalog_updates(catalog_path=Mock())

        assert artifact_updates == [library_update, plugin_update]
        assert resolver_mock.resolve.call_count == 2
        expected_load_metadata_calls = [
            call(
                artifacts=[library],
                repositories=[library_repository],
                verbose=False,
            ),
            call(
                artifacts=[plugin],
                repositories=[plugin_repository],
                verbose=False,
            ),
        ]
        assert load_metadata_mock.call_args_list == expected_load_metadata_calls

    @staticmethod
    def _create_resolver_mock(resolution: UpdateResolution, update: Optional[ArtifactUpdate] = None) -> Mock:
        resolver_mock = Mock()
        resolver_mock.resolve.return_value = (resolution, update)
        return resolver_mock

    @staticmethod
    def _create_catalog_updater(
        library_repositories: Optional[List[Repository]] = None,
        plugin_repositories: Optional[List[Repository]] = None,
        update_resolvers: Optional[List[UpdateResolver]] = None,
        verbose: bool = False,
    ) -> CatalogUpdater:
        if not library_repositories:
            library_repositories = []
        if not plugin_repositories:
            plugin_repositories = []
        if not update_resolvers:
            update_resolvers = []

        return CatalogUpdater(
            library_repositories=library_repositories,
            plugin_repositories=plugin_repositories,
            update_resolvers=update_resolvers,
            verbose=verbose,
        )



import sys
from pathlib import Path
from typing import Dict, Union

from kataloger import package_name
from kataloger.exceptions.kataloger_parse_exception import KatalogerParseException


def remove_suffix(string: str, suffix: str) -> str:
    if sys.version_info < (3, 9):
        if suffix and string.endswith(suffix):
            return string[:-len(suffix)]

        return string

    return string.removesuffix(suffix)


def get_package_file(filename: str) -> Path:
    if sys.version_info < (3, 9):
        from importlib_resources import as_file, files
    else:
        from importlib.resources import as_file, files

    with as_file(files(package_name).joinpath(filename)) as path:
        return path


def load_toml(path: Path) -> Dict[str, Union[str, Dict]]:
    if sys.version_info < (3, 11):
        import tomli as tomllib
        from tomli import TOMLDecodeError
    else:
        import tomllib
        from tomllib import TOMLDecodeError

    with Path.open(path, mode="rb") as file:
        try:
            return tomllib.load(file)
        except TOMLDecodeError as parse_error:
            message = f"Can't parse TOML in \"{path.name}\"."
            raise KatalogerParseException(message) from parse_error



from dataclasses import dataclass
from pathlib import Path
from typing import Optional

from kataloger.data.configuration_data import ConfigurationData


@dataclass(frozen=True)
class KatalogerArguments:
    configuration_path: Optional[Path]
    configuration_data: ConfigurationData
